package grpc
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Module Grpc.BufferSource
t represents a buffer which is based on bytes but keeps track of the length of valid data inside it. The internal capacity is increased when needed.
c ~capacity () creates a new buffer with internal capacity capacity.
copy_from_bigstringaf ~src_off ~src ~dst ~length copies data from src into dst starting from src_off and ending at src_off + length to the end of the buffer.
sub ~start ~length t creates a new buffer from the current, containing the data in the range [start, start+length).
get_u32_be ~pos t returns the unsigned 32 bit big endian integer at pos in t.